#ifndef RCD_TEST_EXT_H
#define RCD_TEST_EXT_H 1

// see rcdt_largefile_op_removed_from_musl
#define _LARGEFILE_SOURCE 1
#define _FILE_OFFSET_BITS 64
#include <fcntl.h>

#include "ruby.h"

#endif /* RCD_TEST_EXT_H */
